//+------------------------------------------------------------------+
//|                                                  Ichimoku EA.mq5 |
//|                        Copyright 2017, MetaQuotes Software Corp. |
//|                                           http://wmua.ru/slesar/ |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, MetaQuotes Software Corp."
#property link      "http://wmua.ru/slesar/"
#property version   "1.000"
#property description "Expert Advisor for Ichimoku indicator signals"
//---
#include <Trade\PositionInfo.mqh>
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>  
#include <Trade\AccountInfo.mqh>
#include <Trade\DealInfo.mqh>
#include <Trade\OrderInfo.mqh>
#include <Expert\Money\MoneyFixedMargin.mqh>
CPositionInfo  m_position;                      // trade position object
CTrade         m_trade;                         // trading object
CSymbolInfo    m_symbol;                        // symbol info object
CSymbolInfo    m_ich_symbol;                    // symbol info object
CAccountInfo   m_account;                       // account info wrapper
CDealInfo      m_deal;                          // deals object
COrderInfo     m_order;                         // pending orders object
CMoneyFixedMargin m_money;
//--- input parameters
sinput string           _0_               = "Ichimoku settings";  // Ichimoku settings
input int               Ich_tenkan_sen    = 9;                    // Ichimoku: period of Tenkan-sen 
input int               Ich_kijun_sen     = 26;                   // Ichimoku: period of Kijun-sen 
input int               Ich_senkou_span_b = 52;                   // Ichimoku: period of Senkou Span B 
input string            Ich_symbol        = "";                   // Ichimoku: symbol  
input ENUM_TIMEFRAMES   Ich_period        = PERIOD_CURRENT;       // Ichimoku: timeframe 
sinput string           _1_               = "Trade settings";     // Trade settings
input uchar             InpBarDisplay     = 1;                    // Bar for value display
input ushort            InpStopLoss       = 50;                   // Stop Loss (in pips)
input ushort            InpTakeProfit     = 50;                   // Take Profit (in pips)
input ushort            InpTrailingStop   = 0;                    // Trailing Stop (if == "0" -> not trailing) (in pips)
input ushort            InpTrailingStep   = 5;                    // Trailing Step (values > "0")(in pips)
input double            Risk              = 5;                    // Risk in percent for a deal from a free margin
//---
ulong          m_magic=787722;               // magic number
ulong          m_slippage=30;                // slippage

double         ExtStopLoss=0;
double         ExtTakeProfit=0;
double         ExtTrailingStop=0;
double         ExtTrailingStep=0;

string         ich_symbol="";
int            ich_digits=0;

int            handle_iIchimoku;             // variable for storing the handle of the iIchimoku indicator 
double         m_adjusted_point;             // point value adjusted for 3 or 5 points
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   if(!m_symbol.Name(Symbol())) // sets symbol name
      return(INIT_FAILED);
   RefreshRates();
//---
   m_trade.SetExpertMagicNumber(m_magic);
//---
   if(IsFillingTypeAllowed(SYMBOL_FILLING_FOK))
      m_trade.SetTypeFilling(ORDER_FILLING_FOK);
   else if(IsFillingTypeAllowed(SYMBOL_FILLING_IOC))
      m_trade.SetTypeFilling(ORDER_FILLING_IOC);
   else
      m_trade.SetTypeFilling(ORDER_FILLING_RETURN);
//---
   m_trade.SetDeviationInPoints(m_slippage);
//--- tuning for 3 or 5 digits
   int digits_adjust=1;
   if(m_symbol.Digits()==3 || m_symbol.Digits()==5)
      digits_adjust=10;
   m_adjusted_point=m_symbol.Point()*digits_adjust;

   ExtStopLoss=InpStopLoss*m_adjusted_point;
   ExtTakeProfit=InpTakeProfit*m_adjusted_point;
   ExtTrailingStop=InpTrailingStop*m_adjusted_point;
   ExtTrailingStep=InpTrailingStep*m_adjusted_point;
//---
   if(!m_money.Init(GetPointer(m_symbol),Period(),m_symbol.Point()*digits_adjust))
      return(INIT_FAILED);
   m_money.Percent(Risk);
//--- create handle of the indicator iIchimoku
   if(!m_symbol.Name(Ich_symbol)) // sets symbol name
     {
      Print("The parameter \"Ichimoku: symbol\" is invalid. The current symbol is assigned");
      ich_symbol=m_symbol.Name();
      ich_digits=m_symbol.Digits()+1;
     }
   else
     {
      ich_symbol=Ich_symbol;
      ich_digits=m_ich_symbol.Digits()+1;
     }

   handle_iIchimoku=iIchimoku(ich_symbol,Ich_period,Ich_tenkan_sen,Ich_kijun_sen,Ich_senkou_span_b);
//--- if the handle is not created 
   if(handle_iIchimoku==INVALID_HANDLE)
     {
      //--- tell about the failure and output the error code 
      PrintFormat("Failed to create handle of the iIchimoku indicator for the symbol %s/%s, error code %d",
                  ich_symbol,
                  Ich_period,
                  GetLastError());
      //--- the indicator is stopped early 
      return(INIT_FAILED);
     }
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---

  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//+------------------------------------------------------------------+
//| Get value of buffers for the iIchimoku                           |
//|  the buffer numbers are the following:                           |
//|   0 - TENKANSEN_LINE, 1 - KIJUNSEN_LINE, 2 - SENKOUSPANA_LINE,   |
//|   3 - SENKOUSPANB_LINE, 4 - CHIKOUSPAN_LINE                      |
//+------------------------------------------------------------------+
   string text="";
   text="Tenkan-sen("+IntegerToString(Ich_tenkan_sen)+") "+
        DoubleToString(iIchimokuGet(TENKANSEN_LINE,InpBarDisplay),ich_digits)+"\n";
   text=text+"Kijun-sen("+IntegerToString(Ich_kijun_sen)+") "+
        DoubleToString(iIchimokuGet(KIJUNSEN_LINE,InpBarDisplay),ich_digits)+"\n";
   text=text+"Senkou Span A "+
        DoubleToString(iIchimokuGet(SENKOUSPANA_LINE,InpBarDisplay),ich_digits)+"\n";
   text=text+"Senkou Span B("+IntegerToString(Ich_senkou_span_b)+") "+
        DoubleToString(iIchimokuGet(SENKOUSPANB_LINE,InpBarDisplay),ich_digits)+"\n";
   text=text+"Chikou Span "+
        DoubleToString(iIchimokuGet(CHIKOUSPAN_LINE,InpBarDisplay),ich_digits);
   Comment(text);
  }
//+------------------------------------------------------------------+
//| TradeTransaction function                                        |
//+------------------------------------------------------------------+
void OnTradeTransaction(const MqlTradeTransaction &trans,
                        const MqlTradeRequest &request,
                        const MqlTradeResult &result)
  {
//---

  }
//+------------------------------------------------------------------+
//| Refreshes the symbol quotes data                                 |
//+------------------------------------------------------------------+
bool RefreshRates(void)
  {
//--- refresh rates
   if(!m_symbol.RefreshRates())
     {
      Print("RefreshRates error");
      return(false);
     }
//--- protection against the return value of "zero"
   if(m_symbol.Ask()==0 || m_symbol.Bid()==0)
      return(false);
//---
   return(true);
  }
//+------------------------------------------------------------------+ 
//| Checks if the specified filling mode is allowed                  | 
//+------------------------------------------------------------------+ 
bool IsFillingTypeAllowed(int fill_type)
  {
//--- Obtain the value of the property that describes allowed filling modes 
   int filling=m_symbol.TradeFillFlags();
//--- Return true, if mode fill_type is allowed 
   return((filling & fill_type)==fill_type);
  }
//+------------------------------------------------------------------+
//| Get value of buffers for the iIchimoku                           |
//|  the buffer numbers are the following:                           |
//|   0 - TENKANSEN_LINE, 1 - KIJUNSEN_LINE, 2 - SENKOUSPANA_LINE,   |
//|   3 - SENKOUSPANB_LINE, 4 - CHIKOUSPAN_LINE                      |
//+------------------------------------------------------------------+
double iIchimokuGet(const int buffer,const int index)
  {
   double Ichimoku[1];
//--- reset error code 
   ResetLastError();
//--- fill a part of the iIchimoku array with values from the indicator buffer that has 0 index 
   if(CopyBuffer(handle_iIchimoku,buffer,index,1,Ichimoku)<0)
     {
      //--- if the copying fails, tell the error code 
      PrintFormat("Failed to copy data from the iIchimoku indicator, error code %d",GetLastError());
      //--- quit with zero result - it means that the indicator is considered as not calculated 
      return(0.0);
     }
   return(Ichimoku[0]);
  }
//+------------------------------------------------------------------+
